#!/usr/bin/env python

import sys
import getopt

import psycopg2 as dbapi

from datetime import timedelta

try:
    # Python >= 2.5 includes SQLite 3
    import sqlite3 as db
except ImportError:
    # installed separately in earlier versions
    from pysqlite2 import dbapi2 as db

from itertools import izip

DEFAULT_PORT = 5432

# The relations to be copied, ordered
# topological not to violate foreign key constraints.
RELATIONS = [
    "master_tbl",
    "ka_status_tbl",
    "rg_kompetenzfestellung_tbl",
    "rg_angebote_bildenden_bereich_tbl",
    "rg_angebote_berufsvorbereitung_tbl",
    "rg_angebote_berufliche_qualifizierung_tbl",
    "rg_angebote_lebensbewaeltigung_tbl"
]

SELECT_TMPL = "SELECT * FROM %(relation)s"
INSERT_TMPL = "INSERT INTO %(relation)s (%%(fields)s) VALUES (%%(placeholders)s)"

SELECT_LOOGBOOK = '''
SELECT id, master_id, datum, bearbeiter, art, dauer
FROM ka_logbuch_tbl
'''

INSERT_LOGBOOK = '''
INSERT INTO ka_logbuch_tbl (id, master_id, datum, bearbeiter, art, dauer)
       VALUES (%(id)s, %(master_id)s, %(datum)s, %(bearbeiter)s, %(art)s, %(dauer)s)
'''

SELECT_DATADUMP = "SELECT value FROM dbinfo WHERE key = 'datadump'"
UPDATE_DATADUMP = "UPDATE dbinfo SET value = %(value)s WHERE key = 'datadump'"

SELECT_TAGS = '''
SELECT id, label, description, systemtag FROM tags
'''

INSERT_TAGS = '''
INSERT INTO tags (id, label, description, systemtag)
       VALUES (%(id)s, %(label)s, %(description)s, %(systemtag)s)
'''

SELECT_NM_TAGS_CASES = '''
SELECT id, master_id, tag_id, created FROM nm_tags_cases
'''

INSERT_NM_TAGS_CASES = '''
INSERT INTO nm_tags_cases (id, master_id, tag_id, created)
       VALUES (%(id)s, %(master_id)s, %(tag_id)s, %(created)s)
'''

def usage(msg = "", exit_code = 0):
    print >> sys.stderr, """%s
usage:
%s -i <sqlite db> -i <postgres db> [-p <port>]
""" % (msg, sys.argv[0])
    sys.exit(exit_code)


def copy_relation(relation, in_cur, out_cur):

    relation = { 'relation': relation }

    select_stmnt = SELECT_TMPL % relation
    insert_tmpl  = INSERT_TMPL % relation

    stmnt, fields = None, []

    in_cur.execute(select_stmnt)

    while True:
        row = in_cur.fetchone()
        if not row: break

        if stmnt is None:
            placeholders, values = [], {}
            for idx, x in enumerate(row):
                name = in_cur.description[idx][0]
                values[name] = x
                fields.append(name)
                placeholders.append("%%(%s)s" % name)
            stmnt = insert_tmpl % {
                'fields'      : ','.join(fields),
                'placeholders': ','.join(placeholders) }
        else:
            values = dict(izip(fields, row))

        out_cur.execute(stmnt, values)

def copy_datadump(in_cur, out_cur):

    print >> sys.stderr, "copy datadump"

    in_cur.execute(SELECT_DATADUMP)
    while True:
        row = in_cur.fetchone()
        if not row: break
        out_cur.execute(UPDATE_DATADUMP, { 'value': row[0] })

def copy_logbook(in_cur, out_cur):

    print >> sys.stderr, "copy ka_logbuch_tbl"

    in_cur.execute(SELECT_LOOGBOOK)
    while True:
        row = in_cur.fetchone()
        if not row: break
        id, master_id, datum, bearbeiter, art, dauer = row

        if dauer is not None:
            dauer = timedelta(seconds = dauer)
        out_cur.execute(INSERT_LOGBOOK, {
            'id'        : id,
            'master_id' : master_id,
            'datum'     : datum,
            'bearbeiter': bearbeiter,
            'art'       : art,
            'dauer'     : dauer
        })

def copy_tags(in_cur, out_cur):

    print >> sys.stderr, "copy tags"

    in_cur.execute(SELECT_TAGS)

    while True:
        row = in_cur.fetchone()
        if not row: break
        
        id, label, description, systemtag = row
        if systemtag == 1:
            systemtag = 'True'
        else:
            systemtag = 'False'
        out_cur.execute(INSERT_TAGS, {
            'id'            : id, 
            'label'         : label, 
            'description'   : description, 
            'systemtag'     : systemtag
        })
    
def copy_nm_tags_cases(in_cur, out_cur):

    print >> sys.stderr, "copy nm_tags_cases"

    in_cur.execute(SELECT_NM_TAGS_CASES)

    while True:
        row = in_cur.fetchone()
        if not row: break
        
        id, master_id, tag_id, created = row
        out_cur.execute(INSERT_NM_TAGS_CASES, {
            'id'        : id, 
            'master_id' : master_id, 
            'tag_id'    : tag_id, 
            'created'   : created
        })

def main():

    indb  = None
    outdb = None
    port  = DEFAULT_PORT

    try:
        opts, args = getopt.getopt(sys.argv[1:], "i:o:p:", [
            "in=", "out=", "port=" ])
    except getopt.GetoptError, err:
        usage(str(err), 1)

    for o, a in opts:
        if o in ("i", "-i", "--in"):
            indb = a
        elif o in ("o", "-o", "--out"):
            outdb = a
        elif o in ("p", "-p", "--port"):
            port = int(a)

    if not indb : usage("error: missing input db",  1)
    if not outdb: usage("error: missing output db", 1)
    if not port : usage("error: missing port",      1)

    in_con, in_cur, out_con, out_cur = None, None, None, None
    try:
        in_con = db.connect(indb)
        in_cur = in_con.cursor()

        out_con = dbapi.connect(database = outdb, port = port)
        out_cur = out_con.cursor()

        try:
            for relation in RELATIONS:
                copy_relation(relation, in_cur, out_cur)
            copy_logbook(in_cur, out_cur)
            copy_datadump(in_cur, out_cur)
            copy_tags(in_cur, out_cur)
            copy_nm_tags_cases(in_cur, out_cur)
            out_con.commit()
        except:
            out_con.rollback()
            raise
    finally:
        for c in (in_cur, in_con, out_cur, out_cur):
            if c:
                try: c.close()
                except: pass

if __name__ == '__main__':
    main()
