#!/usr/bin/env python

import psycopg2 as dbapi

import sys
import getopt
import traceback

SELECT_FIELDS = """
SELECT
    id,
    erstgespraech,
    art_beendigung,
    zeitraum_nachbetreuung,
    kompetenzfeststellung,
    (bildender_bereich = 1 OR 
    bereich_berufliche_qualifizierung = 1 OR
    bereich_berufsvorbereitung = 1 OR
    bereich_lebensbewaeltigung = 1 OR
    sonstiger_foerderbedarf = 1) as in_cm
FROM master_tbl
"""

UPDATE_PHASE = """
UPDATE master_tbl SET phase = %(phase)s WHERE id = %(id)s
"""

UNKNOWN        = -1
CLEAR_START    =  0
CLEAR_ENDE     =  1
CM_START       =  2
CM_ENDE        =  3
NB_START       =  4
NB_ENDE        =  5
BERATUNG_START =  6

def tolerant_close(c):
    if c:
        try: c.close()
        except: pass

def set_phases_db(db_name, user_name, counts, dry, db_port):
    cur, con = None, None

    try:
        con = dbapi.connect(database = db_name, user = user_name, port = db_port)
        cur = con.cursor()

        phases = {
            UNKNOWN       : [],
            CLEAR_START   : [],
            CLEAR_ENDE    : [],
            CM_START      : [],
            CM_ENDE       : [],
            NB_START      : [],
            NB_ENDE       : [],
            BERATUNG_START: []}

        cur.execute(SELECT_FIELDS)

        while True:
            row = cur.fetchone()
            if not row: break

            mid, erstgespraech, art_beendigung, \
            zeitraum_nachbetreuung, kompetenzfeststellung, in_cm = row

            phase = UNKNOWN

            if in_cm:
                phase = CM_START
                if art_beendigung != -1:
                    phase = CM_ENDE
                    if art_beendigung in (0, 2) and zeitraum_nachbetreuung != -1:
                        phase = NB_ENDE
            else:
                if kompetenzfeststellung == -1:
                    phase = CLEAR_START
                    if erstgespraech is None:
                        phase = UNKNOWN 
                elif kompetenzfeststellung == 0:
                    phase = BERATUNG_START
                else:
                    phase = CLEAR_ENDE

            # Erster Ansatz zum setzen der Phase. 
            #if not erstgespraech is None:
            #	phase = CLEAR_START
            #    if kompetenzfeststellung != -1:
            #        phase = CLEAR_ENDE
            #        if in_cm:
            #            phase = CM_START
            #            if art_beendigung != -1:
            #                phase = CM_ENDE
            #                if art_beendigung in (0, 2) and zeitraum_nachbetreuung != -1:
            #                    phase = NB_ENDE
            #        else:
            #            phase = BERATUNG_START
            #else:
            #    pass
            #    #if in_cm:
            #    #    phase = CM_START
            #    #else:
            #    #    phase = BERATUNG_START

            phases[phase].append(mid)

        if not dry:
            try:
                for p, ids in phases.iteritems():
                    d = { 'phase': p }
                    for mid in ids:
                        d['id'] = mid
                        cur.execute(UPDATE_PHASE, d)
                con.commit()
            except:
                con.rollback()
                raise

        for p, ids in phases.iteritems():
            counts[p] = counts.get(p, 0) + len(ids)


    finally:
        tolerant_close(cur)
        tolerant_close(con)

def load_db_names(file_name):
    f = open(file_name)
    try:
        db_names = []
        while True:
            line = f.readline()
            if not line: break
            line = line.strip()
            if not line or line[0] == "#":
                continue
            db_names.append(line)
        return db_names
    finally:
        tolerant_close(f)

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "d:u:f:t", (
            "database=",
            "user=",
            "file=",
            "dry"))
    except getopt.GetoptError, err:
        sys.stderr.write("error: %s\n" % str(err))
        sys.exit(1)

    file_name = None
    db_name   = None
    db_port   = 5432 
    user_name = "postgres"
    dry       = False

    for o, a in opts:
        if o in ("d", "-d", "--database"):
            db_name = a
        elif o in ("u", "-u", "--user"):
            user_name = a
        elif o in ("f", "-f", "--file"):
            file_name = a
        elif o in ("t", "-t", "--dry"):
            dry = True
        elif o in ("p", "-p", "--port"):
            db_port = a

    if not db_name and not file_name:
        print >> sys.stderr, "Need DB name"
        sys.exit(1)

    counts = {}
    db_names = file_name and load_db_names(file_name) or [db_name]

    for db_name in db_names:
        try:
            set_phases_db(db_name, user_name, counts, dry, db_port)
        except:
            traceback.print_exc(file = sys.stderr)
            print >> sys.stderr, "trouble with: %s"  % db_name

    for k in sorted(counts.iterkeys()):
        print "%d: %d" % (k, counts[k])


if __name__ == '__main__':
    main()

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
